// DPF2XML v1.2 - MegaStorms Systems (c) 2007 - Roberto Prieto
// Export DPF to XML
// Import XML to DPF
// Support: images, tiles, fonts and sprites blocks
// If the block already exists, it will be replaced with the new one
// Win32 & Linux versions
// ToDO: sound,music,files support. Buttons?

#include <CRM32Pro.h>
#include <time.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#ifdef _WINDOWS
 #include <direct.h>
 #include <io.h>

#elif defined(_LINUX)
  #include <unistd.h>  
  // External definition(CRM32Pro.so) for Linux machines
  extern char *_strlwr(char *string);
  extern void _splitpath(const char* path, char* drv, char* dir, char* name, char* ext);
  extern void _mkdir(const char *dirname);
#endif

#define DIR_SLASH "/"
#define DPF2XML_VERSION "1.2"
int DPF2XML(char *DPFfilename);
int XML2DPF(char *XMLfilename);
int RemoveBlock(int idDPF,char *type, char *name);

int main(int argc, char *argv[])
{
	char              strExtension[32];

	// Parameter parser (we only need one)
    printf("------------------------------------------------\n");
	printf("DPF2XML - Export and Import DPF to/from XML v%s\n",DPF2XML_VERSION);
	printf("   MegaStorm Systems (c) 2007 - CRM32Pro SDK\n");
	printf("------------------------------------------------\n\n");
	if(argc < 2)
	{
		printf(" - One parameter is needed:\n    + a DPF file, to be exported to an XML\n    + a XML file, to be imported into a DPF\n");
		puts("\nPress [ENTER] key to continue");
	    getchar();
		return 0;
	}
	_splitpath(argv[1], NULL,NULL,NULL,strExtension);
	strcpy(strExtension,_strlwr((char*)strExtension));

	// CRM32Pro init at minimum level
	CRM32Pro.Init(0);

	// Depending on the parameter extension, we proceed with import or export the DPF
	if(!strcmp(strExtension,".dpf")) 
	{
		printf(" - Starting process...\n");
		if(!DPF2XML(argv[1])) printf("\n - Error! the DPF was not exported.\n");
		else printf("\n - DPF exported successfully to a XML + gfx resources.\n");
	}
	else if(!strcmp(strExtension,".xml"))
	{
		printf(" - Starting process...\n");
		if(!XML2DPF(argv[1])) printf("\n - Error! the XML was not imported.\n");
		else printf("\n - XML imported successfully into a new DPF.\n");
	}
	else
	{
		printf(" - Error: input file it is not supported. Try with a DPF or XML file\n");
	}

	// End of execution 
	CRM32Pro.Quit();
	return 0;
}

// Export resources from a DPF to XML and a folder with all BMP/PNG
// .Supports images, tiles, fonts and sprites.
int DPF2XML(char *strFilename)
{
	SDL_Surface      *surface;
	CRM32Pro_CFont   *fontUpdate;
	CRM32Pro_CSprite *spriteUpdate;
	CRM32Pro_CTile   *tileUpdate;
	DPF_BlockData     dpfBlock;
	int               status = 0, idXML = 0, retvalue = 0;
	int               totalBlocks = 0;
	int               processBlocks = 0;
	int               fontBlocks = 0,spriteBlocks = 0,imageBlocks = 0,tileBlocks = 0;
	int               idDPF, ret = 0;
	char              szFileName[128];
	char              szFileXML[128];
	char              szSurface[128];
	char              szBlockName[32];
	int               iExport = 0; 
    long              ltime;
    const struct tm   *localt;

	// Open DPF and init
	idDPF = IStuffDPF->Open(strFilename);
	if(idDPF == 0)
	{
		printf(" - Error: could not open the given DPF.\n");
		return ret;
	}
	totalBlocks = IStuffDPF->GetNumBlocks(idDPF);
	if(totalBlocks == 0) 
	{
		printf(" - Error: could not access to the given DPF. Is it password protected?\n");
		IStuffDPF->Close(idDPF);
		return ret;
	}
	totalBlocks = totalBlocks - IStuffDPF->GetNumDelBlocks(idDPF);

	// Read the first block
	status = IStuffDPF->FindFirst(idDPF,&dpfBlock);

	// If there is a block: create the folder for the BMP/PNG and the XML
	if(status == 1)
	{
		strncpy(szFileName,strFilename,strlen(strFilename)-4);
		szFileName[strlen(strFilename)-4] = '\0';
		strcpy(szFileXML,szFileName);
		strcat(szFileXML,".xml");
		idXML = CRM32Pro.XMLCreate("DPF_file");
		if(idXML == 0) status = -1;
		retvalue = CRM32Pro.XMLAttributeSet(idXML,"source",strFilename);
		if(retvalue == 0) status = -1;		
		time(&ltime);
        localt = localtime(&ltime);
		retvalue = CRM32Pro.XMLAttributeSet(idXML,"date",asctime(localt));
		retvalue = CRM32Pro.XMLAttributeSet(idXML,"dpf2xml_version",DPF2XML_VERSION);
		_mkdir(szFileName);		
	}

	// Loop through each block doing the export
	while(status == 1)
    {
      // ----------------------------Exporting images type = IMG----------------------------
	  if(!strncmp("IMG",dpfBlock.type,3))
	  {
		  // Load the surface
		  surface = IImage->Load(idDPF,dpfBlock.name);
		  // Write in the XML
		  retvalue = CRM32Pro.XMLNodeCreate(idXML,"image"); // Create a new node and point to it.
		  strcpy(szBlockName,"img_");
		  strcat(szBlockName,dpfBlock.name);		  
		  strcpy(szSurface,szFileName);
		  strcat(szSurface,DIR_SLASH);
		  strcat(szSurface,szBlockName);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"name",dpfBlock.name);

		  // Alpha per-pixel?
		  if((surface->flags & SDL_SRCALPHA) && (surface->format->alpha == 255))
		  {
			  strcat(szSurface,".png");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-pixel");		  
			  IImage->SavePNG(szSurface,surface); 
		  }
		  // Alpha per-surface
		  else
		  {
			  strcat(szSurface,".bmp");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-surface");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphavalue",(int)surface->format->alpha);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"colorkey",(int)surface->format->colorkey);
			  IImage->SaveBMP(szSurface,surface);
		  }
		 
		  // Free the surface
		  SDL_FreeSurface(surface);
		  imageBlocks++;
		  iExport++;
		  CRM32Pro.XMLNodeParent(idXML); // Point to father node
	  }

	  // ----------------------------Exporting tiles type = TILE | TILE2----------------------------
	  else if(!strncmp("TILE",dpfBlock.type,4))
	  {
		  tileUpdate = new CRM32Pro_CTile;
		  // Load the tile
		  tileUpdate->Load(idDPF,dpfBlock.name);
		  surface = tileUpdate->GetSurface();
		  // Write in the XML
		  retvalue = CRM32Pro.XMLNodeCreate(idXML,"tile"); // Create a new node and point to it.
		  strcpy(szBlockName,"tile_");
		  strcat(szBlockName,dpfBlock.name);		  
		  strcpy(szSurface,szFileName);
		  strcat(szSurface,DIR_SLASH);
		  strcat(szSurface,szBlockName);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"name",dpfBlock.name);

		  // Alpha per-surface?
		  if(tileUpdate->GetAlpha() != 256)
		  {
			  strcat(szSurface,".bmp");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-surface");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphavalue",(int)surface->format->alpha);			  
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"colorkey",(int)surface->format->colorkey);			  
			  IImage->SaveBMP(szSurface,surface);
		  }
		  // Alpha per-pixel?
		  else
		  {
			  strcat(szSurface,".png");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-pixel");		  
			  IImage->SavePNG(szSurface,surface); 
		  }
		  // Positions
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"xposition",tileUpdate->GetPosX());
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"yposition",tileUpdate->GetPosY());
		  // Tileset
		  int x,y;
		  if(tileUpdate->GetOffset(&x,&y))
		  {
              retvalue = CRM32Pro.XMLAttributeSet(idXML,"tileset","enabled");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"xoffset",x);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"yoffset",y);
		  }
		  else retvalue = CRM32Pro.XMLAttributeSet(idXML,"tileset","disabled");

		  // Free the tile
		  delete tileUpdate;
		  tileBlocks++;
		  iExport++;
		  CRM32Pro.XMLNodeParent(idXML); // Point to father node
	  }

	  // ----------------------------Exporting sprites type = SPR | SPR2----------------------------
	  else if(!strncmp("SPR",dpfBlock.type,3))
	  {
		  spriteUpdate = new CRM32Pro_CSprite;
		  // Load the sprite
		  spriteUpdate->Load(idDPF,dpfBlock.name);
		  surface = spriteUpdate->GetSurface();
		  
		  // Write in the XML
          retvalue = CRM32Pro.XMLNodeCreate(idXML,"sprite"); // Create a new node and point to it
		  strcpy(szBlockName,"sprite_");
		  strcat(szBlockName,dpfBlock.name);		  
		  strcpy(szSurface,szFileName);
		  strcat(szSurface,DIR_SLASH);
		  strcat(szSurface,szBlockName);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"name",dpfBlock.name);

		  // Alpha per-surface?
		  if(spriteUpdate->GetAlpha() != 256)
		  {
			  strcat(szSurface,".bmp");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-surface");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphavalue",(int)surface->format->alpha);			  
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"colorkey",(int)surface->format->colorkey);			  
			  IImage->SaveBMP(szSurface,surface);
		  }
		  // Alpha per-pixel
		  else
		  {
			  strcat(szSurface,".png");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-pixel");		  
			  IImage->SavePNG(szSurface,surface); 
		  }
		  // Offset
		  int x=0,y=0;
		  spriteUpdate->GetOffset(&x,&y);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"xoffset",x);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"yoffset",y);

		  // Loop through each animation
		  for(int i=0;i<surface->h/y;i++)
		  {
			  unsigned short type=0,status=0,frames=0,startframe=0,endframe=0;
			  signed short hotx=0,hoty=0;
			  spriteUpdate->GetAnimProperties(i+1,&type,&status,&frames,&hotx,&hoty,&startframe,&endframe);
			  retvalue = CRM32Pro.XMLNodeCreate(idXML,"animation"); // Create a new animation node and point to it
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"id",i);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"type",type);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"status",status);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"frames",frames);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"hotx",hotx);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"hoty",hoty);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"starframe",startframe);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"endframe",endframe);
			  // Loop through each frame
			  for(int j=0;j<frames;j++)
			  {
				  unsigned int ms=0;
				  retvalue = CRM32Pro.XMLNodeCreate(idXML,"frame"); // Create a new frame node and point to it
                  retvalue = CRM32Pro.XMLAttributeSet(idXML,"id",j);
				  spriteUpdate->GetFrameProperties(i+1,j+1,&ms);
				  retvalue = CRM32Pro.XMLAttributeSet(idXML,"ms",(int)ms);
				  CRM32Pro.XMLNodeParent(idXML); // Point to father node
			  }
			  CRM32Pro.XMLNodeParent(idXML); // Point to father node

		  }

		  // Free the sprite
		  delete spriteUpdate;
		  spriteBlocks++;
		  iExport++;
		  CRM32Pro.XMLNodeParent(idXML); // Point to father node
	  }

	  // ----------------------------Exporting fonts type = FONT | FONT2----------------------------
	  else if(!strncmp("FONT",dpfBlock.type,4))
	  {
		  fontUpdate = new CRM32Pro_CFont;
		  // Load the font
		  fontUpdate->Load(idDPF,dpfBlock.name);
		  surface = fontUpdate->GetSurface();
		  // Write in the XML
		  retvalue = CRM32Pro.XMLNodeCreate(idXML,"font"); // Creamos este nuevo nodo y apuntamos a el
		  strcpy(szBlockName,"font_");
		  strcat(szBlockName,dpfBlock.name);		  
		  strcpy(szSurface,szFileName);
		  strcat(szSurface,DIR_SLASH);
		  strcat(szSurface,szBlockName);
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"name",dpfBlock.name);

		  // Alpha per-surface?
		  if(fontUpdate->GetAlpha() != 256)
		  {
			  strcat(szSurface,".bmp");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-surface");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphavalue",(int)surface->format->alpha);			  
			  IImage->SaveBMP(szSurface,surface);
		  }
		  // Alpha per-pixel
		  else
		  {
			  strcat(szSurface,".png");
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"surface",szSurface);
			  retvalue = CRM32Pro.XMLAttributeSet(idXML,"alphablending","per-pixel");		  
			  IImage->SavePNG(szSurface,surface);
		  }
		  // Kerning
		  retvalue = CRM32Pro.XMLAttributeSet(idXML,"kerning",fontUpdate->GetKerning());

		  // Free the font
		  delete fontUpdate;
		  fontBlocks++;
		  iExport++;
		  CRM32Pro.XMLNodeParent(idXML); // Point to father node
	  }	  

	  // Next block
	  status = IStuffDPF->FindNext(idDPF,&dpfBlock);
      processBlocks++;
	  // Info
	  printf(" - Total blocks %d - Exported %d - Images %d - Tiles %d - Sprites %d - Fonts %d\r",
		  totalBlocks,iExport,imageBlocks,tileBlocks,spriteBlocks,fontBlocks);
	}

	// Close DPF
	IStuffDPF->Close(idDPF);

	// Was an error?
	if(status == -1)
	{
		printf(" - Error creating the output XML\n");
		return ret;
	}

	// Save to XML the real number of exported blocks
	retvalue = CRM32Pro.XMLNodePointTo(idXML,1,"DPF_file");
	retvalue = CRM32Pro.XMLAttributeSet(idXML,"totalblocks",iExport);

	// Did we export anything? If not, remove unused files
	if(iExport == 0) rmdir(szFileName);
	else CRM32Pro.XMLSave(idXML,szFileXML);	
	CRM32Pro.XMLClose(idXML);
	
    // Warning message: there were blocks not exported
	if(totalBlocks != iExport) 
	{
		printf("\n - Warning: existing DPF blocks(%d) does not match with exported ones(%d).",totalBlocks,iExport);
		printf("\n - Info: remember that button,sound,music and files blocks are not exported.\n");
	}

	// Return		
	return 1;
}


// Import resources from a XML/folder to a given DPF
// .Support images, tiles, fonts and sprites
int XML2DPF(char *strFilename)
{ 
    SDL_Surface      *surface;
	CRM32Pro_CFont   *fontUpdate;
	CRM32Pro_CSprite *spriteUpdate;
	CRM32Pro_CTile   *tileUpdate;
	int               status = 0, idXML = 0, retvalue = 0, iTmp = 0, iTmp2 = 0;
	int               totalBlocks = 0;
	int               processBlocks = 0;
	int               fontBlocks = 0,spriteBlocks = 0,imageBlocks = 0,tileBlocks = 0;
	int               idDPF, ret = 0;
	char              szDPFFile[128], *szTmp, *szNodeName;
	int               iImport = 0; 
	char              bFlagAnimErr = -1, bFlagFrameErr = -1;

	// Open XML
	idXML = CRM32Pro.XMLOpen(strFilename);
	if(idXML == 0)
	{
		printf(" - Error: could not open the given XML.\n");
		return 0;
	}

	// Point to DPF_file node
	retvalue = CRM32Pro.XMLNodePointTo(idXML,1,"DPF_file");
	if(retvalue == 0) { printf(" - Error parsing the XML (1)\n"); return 0; }

	// Get attributes: name and totalblocks	
	retvalue = CRM32Pro.XMLAttributeGet(idXML,"source",&szTmp);
	strcpy(szDPFFile,szTmp);
	retvalue = CRM32Pro.XMLAttributeGet(idXML,"totalblocks",&totalBlocks);

	// Open or create the DPF
	if((access(szDPFFile,0)) != -1 ) idDPF = IStuffDPF->Open(szDPFFile);
    else idDPF = IStuffDPF->Create(szDPFFile);
	if(idDPF == -1) { printf(" - Error: could not open/create the desired DPF('%s').\n",szDPFFile); CRM32Pro.XMLClose(idXML); return 0; }

	// Loop processing each block
	printf(" - Trying to generate '%s' with %d blocks\n",szDPFFile,totalBlocks);
	retvalue = CRM32Pro.XMLNodeChild(idXML);
	for(;processBlocks<totalBlocks;)
	{
		// Get node name		
		szNodeName = CRM32Pro.XMLNodeGetName(idXML);

		// ----------------------------Importing images----------------------------
	    if(!strncmp("image",szNodeName,5))
	    {
			CRM32Pro.XMLAttributeGet(idXML,"alphablending",&szTmp);

			// Alpha per-pixel?
			if(!strncmp("per-pixel",szTmp,9)) 
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadPNG(szTmp);
			}
			// Alpha per-surface
			else
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadBMP(szTmp);
				CRM32Pro.XMLAttributeGet(idXML,"colorkey",&iTmp);
				if(iTmp != -1) SDL_SetColorKey(surface,SDL_SRCCOLORKEY | SDL_RLEACCEL, iTmp); 
				else { SDL_SetColorKey(surface,0,0); surface->format->colorkey = -1; }
				CRM32Pro.XMLAttributeGet(idXML,"alphavalue",&iTmp);
				if(iTmp == 255)	SDL_SetAlpha(surface,0,255); 	
				else SDL_SetAlpha(surface,SDL_SRCALPHA,iTmp);				
			}
			
			// Save to DPF. Overwrite any previous image with the same name
			CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp);			
			retvalue = IImage->Save(idDPF,szTmp,surface);
			if(retvalue == -1)
			{
				// Remove previous block and try again
				if(RemoveBlock(idDPF,"IMG",szTmp))	retvalue = IImage->Save(idDPF,szTmp,surface);
			}

     	    // Free memory and stats
		    SDL_FreeSurface(surface);
		    if(retvalue > 0) 
			{
				imageBlocks++;
		        iImport++;
			}
		}

		// ----------------------------Importing tiles----------------------------
	    else if(!strncmp("tile",szNodeName,4))
	    {
            // Create a new tile
			tileUpdate = new CRM32Pro_CTile;			
			
			// Alpha per-pixel?
			CRM32Pro.XMLAttributeGet(idXML,"alphablending",&szTmp);
			if(!strncmp("per-pixel",szTmp,9)) 
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadPNG(szTmp);
				tileUpdate->Create(surface,"temp");
				SDL_FreeSurface(surface);
			}
			// Alpha per-surface
			else
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				tileUpdate->Create(szTmp,"temp");
				CRM32Pro.XMLAttributeGet(idXML,"colorkey",&iTmp);
				tileUpdate->SetColorKey(iTmp);
				CRM32Pro.XMLAttributeGet(idXML,"alphavalue",&iTmp);
				tileUpdate->SetAlpha(iTmp);
			}
			// Positions
			CRM32Pro.XMLAttributeGet(idXML,"xposition",&iTmp);
			CRM32Pro.XMLAttributeGet(idXML,"yposition",&iTmp2);
			tileUpdate->SetPosition(iTmp,iTmp2);

			// Tileset?
			CRM32Pro.XMLAttributeGet(idXML,"tileset",&szTmp);
			if(!strncmp("enabled",szTmp,7)) 
			{
				CRM32Pro.XMLAttributeGet(idXML,"xoffset",&iTmp);
				CRM32Pro.XMLAttributeGet(idXML,"yoffset",&iTmp2);
				tileUpdate->SetTileSet(1,iTmp,iTmp2);
			}

			// Save to DPF.Overwrite any previous tile with the same name
			CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp);
			tileUpdate->SetName(szTmp);			
			retvalue = tileUpdate->Save(idDPF);
			if(retvalue == -1)
			{
				// Remove previous block and try again
				if(RemoveBlock(idDPF,"TILE2",szTmp)) retvalue = tileUpdate->Save(idDPF);
			}			

     	    // Free memory and stats		    
			delete tileUpdate;
		    if(retvalue > 0)
			{
                tileBlocks++;
		    	iImport++;
			}
		}

		// ----------------------------Importing fonts----------------------------
	    else if(!strncmp("font",szNodeName,4))
	    {
			// Create a new font
			fontUpdate = new CRM32Pro_CFont;			

			// Alpha per-pixel?
			CRM32Pro.XMLAttributeGet(idXML,"alphablending",&szTmp);
			if(!strncmp("per-pixel",szTmp,9)) 
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadPNG(szTmp);
				fontUpdate->Create(surface);
			}
			// Alpha per-surface
			else
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadBMP(szTmp);	
				fontUpdate->Create(surface);
				CRM32Pro.XMLAttributeGet(idXML,"alphavalue",&iTmp);
				fontUpdate->SetAlpha(iTmp);
			}

			// Kerning
			CRM32Pro.XMLAttributeGet(idXML,"kerning",&iTmp);
			fontUpdate->SetKerning(iTmp);
			
			// Save to DPF.Overwrite any previous font with the same name
			CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp);
			fontUpdate->SetName(szTmp);			
			retvalue = fontUpdate->Save(idDPF,szTmp);
			if(retvalue == -1)
			{
				// Remove previous existing block
				if(RemoveBlock(idDPF,"FONT2",szTmp)) retvalue = fontUpdate->Save(idDPF,szTmp);
			}			

     	    // Free memory and stats
		    delete fontUpdate;
			SDL_FreeSurface(surface);
		    if(retvalue > 0)
			{
                fontBlocks++;
		    	iImport++;
			}
		}

		// ----------------------------Importing sprites----------------------------
	    else if(!strncmp("sprite",szNodeName,6))
	    {
            // Create a new sprite
			spriteUpdate = new CRM32Pro_CSprite;			
			
			// X/Y offsets
			CRM32Pro.XMLAttributeGet(idXML,"xoffset",&iTmp);
			CRM32Pro.XMLAttributeGet(idXML,"yoffset",&iTmp2);

			// Alpha per-pixel?
			CRM32Pro.XMLAttributeGet(idXML,"alphablending",&szTmp);
			if(!strncmp("per-pixel",szTmp,9)) 
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				surface = IImage->LoadPNG(szTmp);
				spriteUpdate->Create(surface,"temp",iTmp,iTmp2);
				SDL_FreeSurface(surface);
			}
			// Alpha per-surfaced
			else
			{
				CRM32Pro.XMLAttributeGet(idXML,"surface",&szTmp);
				spriteUpdate->Create(szTmp,"temp",iTmp,iTmp2);
				CRM32Pro.XMLAttributeGet(idXML,"colorkey",&iTmp);
				spriteUpdate->SetColorKey(iTmp);
				CRM32Pro.XMLAttributeGet(idXML,"alphavalue",&iTmp);
				spriteUpdate->SetAlpha(iTmp);
			}

			// Get the sprite name
			CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp);

			// Loop through each animation and its frames
			bFlagAnimErr = -1;
   		    retvalue = CRM32Pro.XMLNodeChild(idXML); // Point to the first animation on the XML
			if(retvalue != 0)
			{
				int numAnim = spriteUpdate->GetSurface()->h/iTmp2;
                for(int i=0;i<numAnim;)
		    	{
					int idAnim=0,type=0,status=0,frames=0,startframe=0,endframe=0;
					int hotx=0,hoty=0;
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"id",&idAnim);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"type",&type);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"status",&status);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"frames",&frames);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"hotx",&hotx);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"hoty",&hoty);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"starframe",&startframe);
					retvalue = CRM32Pro.XMLAttributeGet(idXML,"endframe",&endframe);
					// Check ID range
					if(idAnim < 0 || idAnim > numAnim) 
					{
						printf("\r - Warning: problem found importing '%s' sprite, AnimID %d is out of range [0,%d].  \n",szTmp,idAnim,numAnim);
					}
					else 
					{
						spriteUpdate->SetAnimProperties(idAnim + 1,type,status,hotx,hoty,startframe,endframe);

						// Loop through frames
						bFlagFrameErr = -1;
						retvalue = CRM32Pro.XMLNodeChild(idXML); // Point to the first animation on the XML
						if(retvalue != 0)
						{
							for(int j=0;j<frames;)
							{
								int ms=0,idFrame;				  		
								retvalue = CRM32Pro.XMLAttributeGet(idXML,"id",&idFrame);						
								retvalue = CRM32Pro.XMLAttributeGet(idXML,"ms",&ms);
								if(idFrame < 0 || idFrame > frames)
								{
									printf("\r - Warning: problem found importing '%s' sprite, AnimID %d - FrameID %d is out of range [0,%d].  \n",szTmp,i,idFrame,frames);
								}
								else spriteUpdate->SetFrameProperties(idAnim + 1,idFrame + 1,ms);
								// Manual counter inc
								j++;
								// Next frame. If an error happens, break the frames loop
								retvalue = CRM32Pro.XMLNodeNext(idXML); 
								if(retvalue == 0)
									// Discard last loop run
									if(j != frames) { bFlagFrameErr = j + 1; j = frames; }
							}
							// Point to parent node (animation)
							CRM32Pro.XMLNodeParent(idXML);
						}					
						else bFlagFrameErr = 0;

						// Did we get a frame error?
						if(bFlagFrameErr != -1) printf("\r - Warning: problem found importing '%s' sprite, AnimID %d - FrameID %d.     \n",szTmp,i,bFlagFrameErr-1);
					}
		
					// Manual counter inc
					i++;
					// Next animation. If an error happens, break the animation loop
					retvalue = CRM32Pro.XMLNodeNext(idXML);
					if(retvalue == 0) 
						// Discard last loop run
						if(i != numAnim) { bFlagAnimErr = i + 1; i = numAnim; }
				}
				// Point to parent node (sprite)
				CRM32Pro.XMLNodeParent(idXML);
			}
			else bFlagAnimErr = 0;			

			// Did we get an animation error?
			if(bFlagAnimErr != -1) printf("\r - Warning: problem found importing '%s' sprite, AnimID %d.     \n",szTmp,bFlagAnimErr-1);

			// Save to DPF.Overwrite any previous sprite with the same name		
			spriteUpdate->SetName(szTmp);			
			retvalue = spriteUpdate->Save(idDPF);
			if(retvalue == -1)
			{
				// Remove previous existing block
				if(RemoveBlock(idDPF,"SPR2",szTmp)) retvalue = spriteUpdate->Save(idDPF);
			}						

     	    // Free memory and stats		    			
		    delete spriteUpdate;
			if(retvalue > 0)
			{
				spriteBlocks++;
		    	iImport++;
			}			
		}

		// Info & stats
		processBlocks = iImport;
	    printf(" - Total blocks %d - Imported %d - Images %d - Tiles %d - Sprites %d - Fonts %d\r",
		  totalBlocks,iImport,imageBlocks,tileBlocks,spriteBlocks,fontBlocks);
		// Next item. If error, break the loop
		retvalue = CRM32Pro.XMLNodeNext(idXML);	
        if(retvalue == 0) processBlocks = totalBlocks;
	}	
	
	// Warning message is there were blocks not imported
	if(totalBlocks != iImport) printf("\n - Warning: existing XML blocks(%d) does not match with imported ones(%d).\n",totalBlocks,iImport);
    // Close the XML,DPF and return
	CRM32Pro.XMLClose(idXML);
	IStuffDPF->Close(idDPF);
	return 1;
}


// Remove the given DPF/type/name
// Return 0 Failed
//        1 Ok
int RemoveBlock(int idDPF,char *type, char *name)
{
	DPF_BlockData block;
	if(idDPF == -1) return 0;
	
	strcpy(block.name,name);
	strcpy(block.type,type);
	return IStuffDPF->Delete(idDPF,&block);
}
